@@ -1,18 +1,17 @@
 # -*- coding: utf-8 -*-
+# cython: language_level=3, always_allow_keywords=True
 
-## Copyright 2008 by LivingLogic AG, Bayreuth/Germany
-## Copyright 2008 by Walter Dörwald
+## Copyright 2009-2014 by LivingLogic AG, Bayreuth/Germany
+## Copyright 2009-2014 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
-## See ll/__init__.py for the license
+## See ll/xist/__init__.py for the license
 
 
-from __future__ import with_statement
-
 __docformat__ = "reStructuredText"
 
-import os, optparse, collections, contextlib
+import os, collections, contextlib
 
 from ll import url
 from ll.xist import xsc, sims
@@ -50,11 +49,11 @@ else:
 	roles.register_generic_role("meth", meth)
 	en.roles["meth"] = "meth"
 
-	class var(nodes.literal):
+	class obj(nodes.literal):
 		pass
 
-	roles.register_generic_role("var", var)
-	en.roles["var"] = "var"
+	roles.register_generic_role("obj", obj)
+	en.roles["obj"] = "obj"
 
 	class exc(nodes.literal):
 		pass
@@ -131,7 +130,7 @@ class backrefsattrs(xsc.Attrs):
 class refnameattrs(xsc.Attrs):
 	class refname(xsc.TextAttr):
 		"""
-		Internal reference to the `name` attribute of an element. On a
+		Internal reference to the ``name`` attribute of an element. On a
 		'target' element, 'refname' indicates an indirect target which may
 		resolve to either an internal or external reference.
 		"""
@@ -168,17 +167,18 @@ class BaseElement(xsc.Element):
 	xmlns = xmlns
 	class Attrs(xsc.Element.Attrs):
 		class ids(xsc.TextAttr):
-			"`id` is a unique identifier, typically assigned by the system."
+			"``id`` is a unique identifier, typically assigned by the system."
 		class names(xsc.TextAttr):
-			"`name` is an identifier assigned in the markup."
+			"``name`` is an identifier assigned in the markup."
 		class dupnames(xsc.TextAttr):
-			"`dupname` is the same as `name`, used when it's a duplicate."
+			"``dupname`` is the same as ``name``, used when it's a duplicate."
 		class source(xsc.TextAttr):
-			"`source` is the name of the source of this document or fragment."
+			"``source`` is the name of the source of this document or fragment."
 		class class_(xsc.TextAttr):
-			"`class` is used to transmit individuality information forward."
+			"``class`` is used to transmit individuality information forward."
 			xmlname = "class"
-	
+
+
 # Root Element
 class document(BaseElement):
 	class Attrs(BaseElement.Attrs):
@@ -334,15 +334,16 @@ class definition_list(BaseElement):
 
 class definition_list_item(BaseElement):
 	def convert(self, converter):
-		with xsc.Frag() as e:
-			if self[term] or self[classifier]:
-				with doc.dt():
-					+xsc.Frag(self[term])
-					if self[classifier]:
-						+xsc.Text(" (")
-						+xsc.Frag(self[classifier]).withsep(", ")
-						+xsc.Text(")")
-			+xsc.Frag(self[definition])
+		with xsc.build():
+			with xsc.Frag() as e:
+				if self[term] or self[classifier]:
+					with doc.dt():
+						+xsc.Frag(self[term])
+						if self[classifier]:
+							+xsc.Text(" (")
+							+xsc.Frag(self[classifier]).withsep(", ")
+							+xsc.Text(")")
+				+xsc.Frag(self[definition])
 		return e.convert(converter)
 
 
@@ -389,19 +390,28 @@ class field_body(BaseElement):
 
 
 class option_list(BaseElement):
-	pass
+	def convert(self, converter):
+		e = doc.dl(self.content)
+		return e.convert(converter)
 
 
 class option_list_item(BaseElement):
-	pass
+	def convert(self, converter):
+		e = self.content
+		return e.convert(converter)
 
 
 class option_group(BaseElement):
-	pass
-
-
-class option(BaseElement):
-	pass
+	def convert(self, converter):
+		e = doc.dt()
+		for o in self[option]:
+			if e:
+				e.append(", ")
+			e2 = doc.lit(doc.option(o[option_string][0].content))
+			for oa in o[option_argument]:
+				e2.append("=", oa.content)
+			e.append(e2)
+		return e.convert(converter)
 
 
 class option_string(BaseElement):
@@ -415,7 +425,9 @@ class option_argument(BaseElement):
 
 
 class description(BaseElement):
-	pass
+	def convert(self, converter):
+		e = doc.dd(self.content)
+		return e.convert(converter)
 
 
 class literal_block(BaseElement):
@@ -562,7 +574,8 @@ class system_message(BaseElement):
 
 	def convert(self, converter):
 		# A warning has already been issued by docutils, we don't have to do anything
-		return xsc.Null 
+		return xsc.Null
+
 
 class raw(BaseElement):
 	class Attrs(BaseElement.Attrs):
@@ -629,11 +642,11 @@ class meth(BaseElement):
 		return e.convert(converter)
 
 
-class var(BaseElement):
+class obj(BaseElement):
 	model = sims.NoElements()
 
 	def convert(self, converter):
-		e = doc.var(self.content)
+		e = doc.obj(self.content)
 		return e.convert(converter)
 
 
@@ -843,7 +856,7 @@ PE_bibliographic_elements = (
 
 PE_structure_model = (topic, sidebar, transition) + PE_section_elements
 
-document.model = sims.Elements((title, subtitle, decoration, docinfo, transition) + PE_structure_model)
+document.model = sims.Elements(*(title, subtitle, decoration, docinfo, transition) + PE_structure_model)
 title.model = sims.ElementsOrText(*PE_inline_elements)
 subtitle.model = sims.ElementsOrText(*PE_inline_elements)
 docinfo.model = sims.Elements(*PE_bibliographic_elements)
@@ -929,8 +942,7 @@ class ReSTConversionWarning(Warning):
 class ReSTConverter(object):
 	def __init__(self):
 		from ll.xist.ns import doc, abbr
-		self.namedrefs = collections.defaultdict()
-		self.namedrefs.default_factory = list
+		self.namedrefs = collections.defaultdict(list)
 		self.unnamedrefs = []
 		self.doc = doc
 		self.abbr = abbr
@@ -1000,16 +1012,18 @@ class ReSTConverter(object):
 			warnings.warn(ReSTConversionWarning(str(node)))
 			return xsc.Null # ignore system messages
 		else:
-			raise TypeError("can't handle %r" % node.__class__)
+			raise TypeError("can't handle {!r}".format(node.__class__))
 
 
 def fromstring(string, base=None, **options):
-	doc = core.publish_doctree(string, source_path=base, settings_overrides=options)
+	realoptions = dict(tab_width=3)
+	realoptions.update(options)
+	doc = core.publish_doctree(string, source_path=base, settings_overrides=realoptions) # This requires docutils
 	elements = globals()
 
 	def toxist(node):
 		if isinstance(node, nodes.Text):
-			return xsc.Text(unicode(node.astext()))
+			return xsc.Text(str(node.astext()))
 		else:
 			e = elements[node.__class__.__name__](toxist(child) for child in node.children)
 			e.startloc = xsc.Location(node.source, node.line)
@@ -1020,8 +1034,6 @@ def fromstring(string, base=None, **options):
 					e.attrs[attrkey] = attrvalue
 			return e
 
-	global x
-	x = doc
 	return toxist(doc)
 
 
@@ -1030,8 +1042,8 @@ def fromstream(stream, base=None, **options):
 
 
 def fromfile(filename, base=None, **options):
-		filename = os.path.expanduser(filename)
-		if base is None:
-			base = filename
-		with contextlib.closing(open(filename, "rb")) as stream:
-			return fromstream(stream, base, **options)
+	filename = os.path.expanduser(filename)
+	if base is None:
+		base = filename
+	with contextlib.closing(open(filename, "rb")) as stream:
+		return fromstream(stream, base, **options)
